using System;
using System.Linq;
using System.ComponentModel;
using System.Threading;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

using ReflectionIT.Minesweeper.Controls;
using System.Windows;
using System.Collections.Generic;
using System.Windows.Threading;

namespace ReflectionIT.Minesweeper {

    /// <summary>
    /// Summary description for Game.
    /// </summary>
    public class Game : INotifyPropertyChanged {

        public event EventHandler GameOver;

        public event PropertyChangedEventHandler PropertyChanged;

        private Square[,] _squares;
        private int _correctMarkedMines;
        private int _incorrectMarkedMines;
        private int _height;
        private int _width;
        private int _time;
        private int _mines;
        private Grid _grid;
        private DispatcherTimer _timer;

        protected virtual void OnPropertyChanged(string propName) {
            if (PropertyChanged != null) {
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
            }
        }

        public void Init(Grid grid, int width, int height, int mines) {
            _grid = grid;
            _width = width;
            _height = height;
            _mines = mines;

            for (int i = 0; i < width; i++) {
                grid.ColumnDefinitions.Add(new ColumnDefinition());
            }

            for (int i = 0; i < height; i++) {
                grid.RowDefinitions.Add(new RowDefinition());
            }

            _timer = new DispatcherTimer();
            _timer.Interval = new TimeSpan(0, 0, 1);
            _timer.Tick += new EventHandler(_timer_Tick);
        }

        void _timer_Tick(object sender, EventArgs e) {
            Time++;
        }

        private void Square_MarkedChanged(object sender, EventArgs e) {
            Square s = (Square)sender;
            if (s.Marked) {
                if (s.Mined) {
                    _correctMarkedMines++;
                } else {
                    _incorrectMarkedMines++;
                }
            } else {
                if (s.Mined) {
                    _correctMarkedMines--;
                } else {
                    _incorrectMarkedMines--;
                }
            }

            //OnMarkedMinesChanged();
            OnPropertyChanged("TotalMarkedMines");

            // Check whether we are finished
            if (Won) {
                foreach (Square sq in _squares) {
                    sq.UnsubsrcibeEvents();
                    sq.Button.Click += delegate { Setup(); };
                }
                OnGameOver();

            }
        }

        public int TotalMarkedMines {
            get { return _mines - _correctMarkedMines - _incorrectMarkedMines; }
        }

        private void Square_Exploded(object sender, EventArgs e) {
            _timer.Stop();
            foreach (Square s in _squares) {
                s.UnsubsrcibeEvents();
                s.Button.Click += delegate { Setup(); };
                if (s.Mined && !s.Marked) {
                    s.Button.Content = new MineImage();
                } else {
                    if (s.Marked && !s.Mined) {
                        s.Button.Content = new InvalidMineImage();
                    }
                }
            }
            OnGameOver();
        }

        public int Height {
            get { return this._height; }
        }

        public bool IsSquareMined(int x, int y) {
            if (x >= 0 && x < Width && y >= 0 && y < Height) {
                return _squares[x, y].Mined;
            }
            // does not exists so NotMined
            return false;
        }

        public int Mines {
            get { return this._mines; }
        }

        protected virtual void OnGameOver() {
            _timer.Stop();
            if (GameOver != null) {
                GameOver(this, EventArgs.Empty);
            }
        }

        public void UncoverSquare(int x, int y) {
            if (x >= 0 && x < Width && y >= 0 && y < Height) {
                _squares[x, y].Uncover();
            }
        }

        public Grid Grid {
            get { return this._grid; }
        }

        public void Setup() {

            (this.Grid.FindName("media2") as MediaElement).Stop();
            (this.Grid.FindName("media") as MediaElement).Stop();

            Time = 0;
            _timer.Stop();

            _correctMarkedMines = 0;
            _incorrectMarkedMines = 0;
            Grid.Children.Clear();

            _squares = new Square[Width, Height];
            for (int x = 0; x < Width; x++) {
                for (int y = 0; y < Height; y++) {
                    Square s = new Square(this, x, y);
                    s.Exploded += new EventHandler(Square_Exploded);
                    s.MarkedChanged += new EventHandler(Square_MarkedChanged);
                    _squares[x, y] = s;
                }
            }
            _squares[0, 0].Button.Focus();

            // Place Mines
            int b = 0;
            Random r = new Random();
            while (b < Mines) {
                int x = r.Next(Width);
                int y = r.Next(Height);

                Square s = _squares[x, y];
                if (!s.Mined) {
                    s.Mined = true;
                    b++;
                }
            }

            OnPropertyChanged("TotalMarkedMines");
        }

        public int Time {
            get { return this._time; }
            set {
                this._time = value;
                OnPropertyChanged("Time");
            }
        }

        public int Width {
            get { return this._width; }
        }

        /// <summary>
        /// Returns true when the player has won the game
        /// </summary>
        public bool Won {
            get { return _correctMarkedMines == Mines; }
        }

        internal void EnableTimer() {
            _timer.Start();
        }

        internal bool RightMouseClick(int x, int y) {
            if (_timer.IsEnabled) {
                var q = System.Windows.Media.VisualTreeHelper.FindElementsInHostCoordinates(new Point(x, y), _grid);
                var b = q.OfType<Button>().FirstOrDefault();
                if (b != null) {
                    var s = _squares.Cast<Square>().FirstOrDefault(sq => sq.Button == b);
                    if (s != null) {
                        s.ClickRight();
                        return true;
                    }
                }
            }
            return false;
        }

    }
}

