using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using ReflectionIT.Minesweeper.Controls;

namespace ReflectionIT.Minesweeper {
    public class Square {

        private static SolidColorBrush[] _colors = new SolidColorBrush[] {
            new SolidColorBrush(Colors.Black),
            new SolidColorBrush(Colors.Blue),
            new SolidColorBrush(Colors.Green),
            new SolidColorBrush(Colors.Red),
            new SolidColorBrush(Colors.Magenta),
            new SolidColorBrush(Colors.Brown),
            new SolidColorBrush(Colors.Yellow),
            new SolidColorBrush(Colors.Orange),
            new SolidColorBrush(Colors.Purple)
        };

        public event EventHandler MarkedChanged;
        public event EventHandler Exploded;

        private Button _button;
        private bool _marked = false;
        private Game _game;
        private bool _mined = false;
        private bool _uncovered = false;
        private int _x;
        private int _y;

        public Square(Game game, int x, int y) {
            _game = game;
            _x = x;
            _y = y;

            double w = _game.Grid.Width / _game.Width;
            double h = _game.Grid.Height / _game.Height;

            _button = new Button();
            _button.FontSize = 20;
            _button.Click += _button_Click;
            _button.Style = Application.Current.Resources["ButtonStyle1"] as Style;

            Grid.SetColumn(_button, x);
            Grid.SetRow(_button, y);
            _game.Grid.Children.Add(_button);

        }

        void _button_Click(object sender, RoutedEventArgs e) {
            if (!Uncovered && Button.IsEnabled) {
                Button_Click(null, null);
            }
        }

        void _button_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            if (!Uncovered && Button.IsEnabled) {
                Button_Click(null, null);
            }
        }

        public Button Button {
            get { return this._button; }
        }

        private void Button_Click(object sender, RoutedEventArgs e) {
            _game.EnableTimer();
            ClickLeft();
        }

        private void ClickLeft() {
            if (!Marked) {
                if (Mined) {
                    this.Explode();
                } else {
                    this.Uncover();
                }
            }
        }

        public void ClickRight() {
            if (!Uncovered && Button.IsEnabled) {
                if (Marked) {
                    Unmark();
                } else {
                    Mark();
                }
                OnMarkedChanged();
            }
        }

        private void Mark() {
            _marked = true;
            Button.Content = new MarkedImage();
        }

        private void Unmark() {
            _marked = false;
            this.Text = "?";
            Button.Foreground = _colors[7];
        }

        public bool Marked {
            get { return this._marked; }
        }

        public bool Mined {
            get { return this._mined; }
            set { this._mined = value; }
        }

        protected void OnMarkedChanged() {
            if (MarkedChanged != null) {
                MarkedChanged(this, EventArgs.Empty);
            }
        }

        protected void OnExploded() {
            if (Exploded != null) {
                Exploded(this, EventArgs.Empty);
            }
        }

        private void Explode() {
            // Explode
            OnExploded();
        }

        public string Text {
            get { return (string)_button.Content; }
            set { _button.Content = value; }
        }

        public void Uncover() {
            if (!Uncovered && !Marked) {
                _uncovered = true;

                // Count Bombs
                int c = 0;
                if (_game.IsSquareMined(X - 1, Y - 1)) c++;
                if (_game.IsSquareMined(X - 0, Y - 1)) c++;
                if (_game.IsSquareMined(X + 1, Y - 1)) c++;
                if (_game.IsSquareMined(X - 1, Y - 0)) c++;
                if (_game.IsSquareMined(X - 0, Y - 0)) c++;
                if (_game.IsSquareMined(X + 1, Y - 0)) c++;
                if (_game.IsSquareMined(X - 1, Y + 1)) c++;
                if (_game.IsSquareMined(X - 0, Y + 1)) c++;
                if (_game.IsSquareMined(X + 1, Y + 1)) c++;

                Button.Foreground = _colors[c];
                if (c > 0) {
                    this.Text = c.ToString();
                } else {
                    //Button.IsEnabled = false;
                    this.Text = string.Empty;

                    Button.AnimateDoubleTo(50, "(UIElement.Opacity)", 0, delegate {
                        Button.IsEnabled = false;

                        _game.UncoverSquare(X - 1, Y - 1);
                        _game.UncoverSquare(X - 0, Y - 1);
                        _game.UncoverSquare(X + 1, Y - 1);
                        _game.UncoverSquare(X - 1, Y - 0);
                        _game.UncoverSquare(X - 0, Y - 0);
                        _game.UncoverSquare(X + 1, Y - 0);
                        _game.UncoverSquare(X - 1, Y + 1);
                        _game.UncoverSquare(X - 0, Y + 1);
                        _game.UncoverSquare(X + 1, Y + 1);

                    });
                }
            }
        }

        public bool Uncovered {
            get { return this._uncovered; }
        }

        public int X {
            get { return this._x; }
        }

        public int Y {
            get { return this._y; }
        }

        public void UnsubsrcibeEvents() {
            _button.Click -= _button_Click;
        }



    }
}

